/*
 * For Epsilon version 6.51:
 * E-Mail commands. Copyright(C) 1991, K. Shane Hartman
 *
 * This file is free software; you can redistribute it and/or modify
 * it under so long as this notice is preserved and no fee is charged 
 * (other than reasonable media fees) for distribution.  You are forbidden
 * to deny these rights to any user of Epsilon.  Lugaru Software Ltd. may
 * incorporate any and all of this code into Epsilon and have all rights to
 * the code, since it is only useful for Epsilon.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * Revision History:
 * Send bug fixes, comments or suggestions to shane@ai.mit.edu.
 *
 * Version 1.0: 07/29/91 shane@ai.mit.edu
 * Version 1.1: 08/02/91 shane@ai.mit.edu Comment and change copyright wording.
 * Version 1.2: 08/05/91 shane@ai.mit.edu Change regex to simpler form.
 * Version 1.3: 08/05/92 shane@ai.mit.edu Port to v6.0
 * Version 1.4: 07/14/93 aaron@home.nointernet.ibmclone (Aaron A. Collins - AAC)
 *									Port to v6.51, PMAIL 2.3, Novell Netware calls.
 *
 * This file implements an interface to public domain email program 
 * PMAIL for Novell Netware.
 *
 * You can obtain PMAIL via anonymous FTP from the following:
 *
 *      RODAN.ACS.SYR.EDU
 *      SPLICER.CBA.HAWAII.EDU
 *
 * It's not bad for free and installs in seconds.  This could be made
 * to work for any mail program which has a decent command line
 * interface.  This program works by using the command line interface
 * to PMAIL:  PMAIL "user-list" file-to-send "subject".
 *
 * The commands implemented are:
 *
 * M-x mail			- Compose a message.
 * M-x send-mail	- Send the message (bound to C-c C-s).
 * M-x show-mail	- Dired on users MHS mail directory.
 * M-x check-mail	- counts new or unread mail in MHS dir.
 */

#include "eel.h"
#include "lowlevel.h"

/*
 * these are NetWare DOS call I/O structures
 */

struct REQ
{
	char len_lo;	/* pkt length (word value) will be 2 */
	char len_hi;
	char cmd;
	char conn;
} request;

/* EEL structure packing is funny.  All entries char due to this. */
struct REP
{
	char len_lo;	/* pkt length (word value) will be 62 */
	char len_hi;
	char id_lo_lo;
	char id_lo_hi;
	char id_hi_lo;
	char id_hi_hi;
	char typex_lo;
	char typex_hi;
	char name[48];
	char time[8];
} reply;

/*
 * Global Disk Transfer Area for DOS find_first, find_next functions.
 */
char dta[44];

/* 
 * This is the default user name.  It gets read in from Netware.
 */
char user_name[16] = "";

/*
 * This is the root path on your server to the MHS mail directory tree.
 * Normally, it will be "Z:\MAIL\".  You may have to map Z: to SYS: to
 * get this to work, but since most users have some search path(s) to
 * SYS: somewhere, this should be pretty easy.  This will do until Epsilon
 * understands Netware Paths like "SYS:MAIL\" in some future version.
 * mail_root _MUST_ end with a "\\"! - AAC
 */
char mail_root[] = "Z:\\MAIL\\";	/* set to your MHS root directory path */
char mail_dir[128] = "";			/* final assembled mail dir pathname */

/*
 * Path to the mailer program - no path required if it's in the SEARCH
 */
char mailer[32] = "pmail.exe";

#undef NULL
#define NULL ((char *) 0)

#ifndef FALSE
#define FALSE 0
#define TRUE !FALSE
#endif

keytable mail_tab;                      /* key table for mail mode */
keytable mail_cx_tab;                   /* for C-x */

/*
 * M-x mail Compose a message.
 */
command mail()
{
  zap ("*Mail*");
  to_buffer ("*Mail*");
  mode_keys = mail_tab;                 /* use these keys */
  mail_tab[CTRL('X')] = find_index ("mail-cx-tab");
  major_mode = "Mail";
  fill_mode = 1;
  try_calling ("mail-hook");
  make_mode ();
  bprintf ("To: \nSubject: \n--Text follows this line--\n");
  say("Type C-x C-s when finished to send the message.");
  point = 4;
}

/*
 * C-x C-s: send composed message.
 */
command send_mail() on mail_cx_tab[CTRL('S')]
{
  char subject[81];
  char to[81];
  int save = point;

  if (strfcmp (bufname, "*Mail*"))
    goto bad_format;
  point = 0;
  if (re_search (1, "To:[ \t]+"))
  {
    if (!parse_string (1, ".+$", to))
      get_string (to, "To: ");
    if (!*to)
    {
      point = save;
      error ("Must specify recipient");
    }
  }
  else
    goto bad_format;
  if (re_search (1, "Subject:[ \t]+"))
  {
    if (!parse_string (1, ".+$", subject))
      get_string (subject, "Subject: ");
  }
  else
    goto bad_format;
  if (search (1, "--Text follows this line--\n"))
  {
    int start = point;
    int end;
    char *dir = getenv ("TMP");
    char tempfile[FNAMELEN];
	if (dir)
	{
		if (dir[strlen(dir) - 1] == '\\')	/* strip trailing "\" from TMP pathspec */
			dir[strlen(dir) - 1] = '\0';
	}
    sprintf (tempfile, "%s%c$draft$", dir ? dir : "", path_sep);
/*  bprintf ("To:       %s\n\n", to);*/
/*	Frigging PMAIL does not do this - PMAIL 2.3 and later does... - AAC */
    end = point;
    point = start;
    /* Put the message in a temporary file */
    if (!write_part (tempfile, 1, point, size ()))
    {
      char cmdline[128];
      
      delete (start, end);

      sprintf (cmdline, "%s %s \"%s\" \"%s\" >NUL", mailer, tempfile, to, subject);
      
      if (!shell ("", cmdline, ""))
      {
        say ("Message sent to %s", to);
        modified = 0;
        to_buffer_num (give_prev_buf ());
		delete_buffer ("*Mail*");  /* AAC - delete mail buffer if sent OK */
      }
      else
      {
        point = save;
        say ("%s lost!", mailer);
      }
      delete_file (tempfile);
    }
  }
  else
    goto bad_format;
  point = save;
  return;
bad_format:
  point = save;  
  error ("Badly formatted message");
}

/*
 * This works by doing a dired in the mail directory for a user.  PMAIL
 * writes mail to this directory: old mail are files with extension .cnr,
 * new mail with extension .cnm.  Rewrite by AAC.
 * This could be made a lot nicer (read the files for subject and sender
 * and display a list of those instead of raw filenames.  This works for
 * me.  If you do make a nice one, please let me have it.
 */
 
/* With arg, shows new mail, otherwise shows all mail */
command show_mail() on mail_cx_tab[CTRL('R')]
{
	char pat[FNAMELEN];
	char *bname = bufname;
	int ret;
	jmp_buf *old_level = top_level;
	jmp_buf this_level;

	get_user_name();
	if (user_name[0])
	{
		top_level = &this_level;
		if (ret = setjmp (top_level))
		{
			char *dired = bufname;

			if (dired != bname)
			{
				bufname = bname;
				delete_buffer (dired);
			}
			top_level = old_level;
			longjmp (top_level, ret);
		}
		/* Change this to your server drive and mail directory */
		sprintf (pat, "%s\\*.cn%c", mail_dir, has_arg ? 'm' : '?');
		dired_one (pat);
		top_level = old_level;
	}
	else error ("I don't know you");
}

/*
 * check_mail simply gets a given user's name and mail path and then
 * counts the number of new or unread mail messages in the user's
 * mailbox. - AAC
 */

/* With arg, checks for new mail, otherwise checks for all mail */
command check_mail() on mail_cx_tab[CTRL('C')]
{
	char pat[FNAMELEN];
	int new_mesgs = 0, carry = FALSE;

	get_user_name();
	if (user_name[0])
	{
		sprintf (pat, "%s\\*.cn%c", mail_dir, has_arg ? 'm' : '?');

		m_regs.b.ah = 0x1A;							/* DOS set DTA function */
		m_regs.w.ds = get_pointer(dta, SEGMENT);	/* get ptr to dummy DTA */
		m_regs.w.dx = get_pointer(dta, !SEGMENT);
		do_interrupt(DOS_SERVICES, &m_regs);

		m_regs.b.ah = 0x4E;							/* DOS find first function */
		m_regs.w.ds = get_pointer(pat, SEGMENT);	/* get ptr to path with wildcards */
		m_regs.w.dx = get_pointer(pat, !SEGMENT);
		do_interrupt(DOS_SERVICES, &m_regs);

		if (!(m_regs.w.flags & CARRYFLAG))			/* if no carry we found a match */
		{
			new_mesgs++;
			while (!carry)
			{
				m_regs.b.ah = 0x4F;						/* DOS find next function */
				m_regs.w.ds = get_pointer(dta, SEGMENT); /* get ptr to dummy DTA */
				m_regs.w.dx = get_pointer(dta, !SEGMENT);
				do_interrupt(DOS_SERVICES, &m_regs);
				if (!(carry = (m_regs.w.flags & CARRYFLAG))) /* no carry = found file */
					new_mesgs++;
			}
		}

		say ("User %s has %d new or unread mail messages.", user_name, new_mesgs);
	}
	else error ("I don't know you");
}

/*
 * get_user_name()  A true Netware-aware function.  This routine gets
 * the proper username from the Netware DOS services extensions.  As
 * I noticed the unique ID number it was returning also happened to
 * be the user's MAIL\XXXXXXX directory name, this now could give us
 * the ability to be a standalone MHS utility, eventually.  Look Ma,
 * No PMAIL! - AAC
 */

get_user_name()		/* this is a little clumsy but it works for me. - AAC */
{
	int mail1, mail2;
	char maildir1[10], maildir2[10];
	int connection, x1, x2, leading;

	m_regs.b.ah = 0xDC;					/* get connection number to AL */

	do_interrupt(DOS_SERVICES, &m_regs);

	connection = m_regs.b.al & 0xFF;	/* conn # returned in AL */
	
	m_regs.b.ah = 0xE3;					/* get user info */

	m_regs.w.ds = get_pointer((char *)&request, SEGMENT);
	m_regs.w.si = get_pointer((char *)&request, !SEGMENT);

	m_regs.w.es = get_pointer((char *)&reply, SEGMENT);
	m_regs.w.di = get_pointer((char *)&reply, !SEGMENT);

	request.len_lo = 2;					/* request buffer is 2 bytes long */
	request.len_hi = 0;
	request.cmd = 22;					/* get user info */
	request.conn = connection;			/* on this connection */

	reply.len_lo = 62;					/* reply buffer is 62 bytes long */
	reply.len_hi = 0;
	reply.id_lo_lo = reply.id_lo_hi = reply.id_hi_lo = reply.id_hi_hi = reply.typex_lo = reply.typex_hi = 0;
	reply.name[0] = 0;					/* terminate user string in case of err */

	do_interrupt(DOS_SERVICES, &m_regs);

	strncpy(user_name, reply.name, 16);	/* take user name directly from buffer */
	strcpy(mail_dir,  mail_root);		/* copy root of mail path */

	mail1 = ((int)reply.id_lo_lo << 8) + reply.id_lo_hi;
	mail2 = ((int)reply.id_hi_lo << 8) + reply.id_hi_hi;

	sprintf(maildir1, "%04X", mail1);
	sprintf(maildir2, "%04X", mail2);

	strcat(maildir1, maildir2);

	leading = TRUE;						/* at start we are on leading chars */
	for (x1 = x2 = 0; x1 < 8; x1++)
	{
		if (leading)					/* if examining leading characters */
		{
			if (maildir1[x1] == '0')	/* don't copy leading 0's in dir name */
				continue;
		}
		leading = FALSE;				/* first nonzero leading character */
		maildir2[x2++] = maildir1[x1];	/* copy other char(s) */
	}
	maildir2[x2] = '\0';				/* tack on terminator when done */

	strcat(mail_dir, maildir2);			/* tack on assembled mail dir name */
}
